@@ -1,12 +1,12 @@
 /*******************************************************************
  * JPEGoptim
- * Copyright (c) Timo Kokkonen, 1996-2014.
+ * Copyright (c) Timo Kokkonen, 1996-2015.
  * All Rights Reserved.
  *
  * requires libjpeg (Independent JPEG Group's JPEG software 
  *                     release 6a or later...)
  *
- * $Id: 954484226e55cf941d2a92891935e3f2912ed6f1 $
+ * $Id: be21505b98c3a351e818bfceb3abaa4115b8153b $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,9 @@
 #include "jpegoptim.h"
 
 
-#define VERSIO "1.4.1"
+#define VERSIO "1.4.3"
+#define COPYRIGHT  "Copyright (c) 1996-2015, Timo Kokkonen"
+
 
 #define LOG_FH (logs_to_stdout ? stdout : stderr)
 
@@ -54,13 +56,14 @@ struct my_error_mgr {
 };
 typedef struct my_error_mgr * my_error_ptr;
 
-const char *rcsid = "$Id: 954484226e55cf941d2a92891935e3f2912ed6f1 $";
+const char *rcsid = "$Id: be21505b98c3a351e818bfceb3abaa4115b8153b $";
 
 
 int verbose_mode = 0;
 int quiet_mode = 0;
 int global_error_counter = 0;
 int preserve_mode = 0;
+int preserve_perms = 0;
 int overwrite_mode = 0;
 int totals_mode = 0;
 int stdin_mode = 0;
@@ -95,6 +98,7 @@ struct option long_options[] = {
   {"force",0,0,'f'},
   {"version",0,0,'V'},
   {"preserve",0,0,'p'},
+  {"preserve-perms",0,0,'P'},
   {"strip-all",0,0,'s'},
   {"strip-none",0,&strip_none,1},
   {"strip-com",0,&save_com,0},
@@ -143,8 +147,7 @@ my_output_message (j_common_ptr cinfo)
 
 void print_usage(void) 
 {
-  fprintf(stderr,PROGRAMNAME " v" VERSIO 
-	  "  Copyright (c) Timo Kokkonen, 1996-2014.\n"); 
+  fprintf(stderr,PROGRAMNAME " v" VERSIO "  " COPYRIGHT "\n");
 
   fprintf(stderr,
 	  "Usage: " PROGRAMNAME " [options] <filenames> \n\n"
@@ -165,8 +168,11 @@ void print_usage(void)
 	  "  -T<threshold>, --threshold=<threshold>\n"
 	  "                    keep old file if the gain is below a threshold (%%)\n"
 	  "  -b, --csv         print progress info in CSV format\n"
-	  "  -o, --overwrite   overwrite target file even if it exists\n"
+	  "  -o, --overwrite   overwrite target file even if it exists (meaningful\n"
+          "                    only when used with -d, --dest option)\n"
 	  "  -p, --preserve    preserve file timestamps\n"
+	  "  -P, --preserve-perms\n"
+          "                    preserve original file permissions by overwriting it\n"
 	  "  -q, --quiet       quiet mode\n"
 	  "  -t, --totals      print totals after processing all files\n"
 	  "  -v, --verbose     enable verbose mode (positively chatty)\n"
@@ -192,7 +198,7 @@ void print_version()
 
   
   printf(PROGRAMNAME " v%s  %s\n",VERSIO,HOST_TYPE);
-  printf("Copyright (c) 1996-2014  Timo Kokkonen.\n");
+  printf(COPYRIGHT "\n");
 
   if (!(err=jpeg_std_error(&jcerr)))
     fatal("jpeg_std_error() failed");
@@ -344,7 +350,7 @@ int main(int argc, char **argv)
   /* parse command line parameters */
   while(1) {
     opt_index=0;
-    if ((c=getopt_long(argc,argv,"d:hm:nstqvfVpoT:S:b",long_options,&opt_index))
+    if ((c=getopt_long(argc,argv,"d:hm:nstqvfVpPoT:S:b",long_options,&opt_index))
 	      == -1) 
       break;
 
@@ -407,6 +413,9 @@ int main(int argc, char **argv)
     case 'p':
       preserve_mode=1;
       break;
+    case 'P':
+      preserve_perms=1;
+      break;
     case 's':
       save_exif=0;
       save_iptc=0;
@@ -480,6 +489,7 @@ int main(int argc, char **argv)
   do {
     if (stdin_mode) {
       infile=stdin;
+      set_filemode_binary(infile);
     } else {
       if (!argv[i][0]) continue;
       if (argv[i][0]=='-') continue;
@@ -512,7 +522,7 @@ int main(int argc, char **argv)
 	  warn("skipping special file: %s",argv[i]); 
 	continue;
       }
-      if ((infile=fopen(argv[i],"r"))==NULL) {
+      if ((infile=fopen(argv[i],"rb"))==NULL) {
 	warn("cannot open file: %s", argv[i]);
 	continue;
       }
@@ -556,7 +566,7 @@ int main(int argc, char **argv)
 
      if (cmarker->marker == EXIF_JPEG_MARKER &&
 	 !memcmp(cmarker->data,EXIF_IDENT_STRING,EXIF_IDENT_STRING_SIZE))
-       strncat(marker_str,"Exiff ",sizeof(marker_str)-strlen(marker_str)-1);
+       strncat(marker_str,"Exif ",sizeof(marker_str)-strlen(marker_str)-1);
 
      if (cmarker->marker == IPTC_JPEG_MARKER)
        strncat(marker_str,"IPTC ",sizeof(marker_str)-strlen(marker_str)-1);
@@ -619,14 +629,13 @@ int main(int argc, char **argv)
      fflush(LOG_FH);
    }
 
-   fclose(infile);
-   infile=NULL;
      
 
    if (dest && !noaction) {
      if (file_exists(newname) && !overwrite_mode) {
        warn("target file already exists: %s\n",newname);
        jpeg_abort_decompress(&dinfo);
+       fclose(infile);
        if (buf) FREE_LINE_BUF(buf,dinfo.output_height);
        continue;
      }
@@ -638,6 +647,7 @@ int main(int argc, char **argv)
      
      jpeg_abort_compress(&cinfo);
      jpeg_abort_decompress(&dinfo);
+     fclose(infile);
      if (!quiet_mode) fprintf(LOG_FH," [Compress ERROR]\n");
      if (buf) FREE_LINE_BUF(buf,dinfo.output_height);
      compress_err_count++;
@@ -735,7 +745,7 @@ int main(int argc, char **argv)
        
      } else {
        int newquality;
-       int dif = round(abs(oldquality-quality)/2.0);
+       int dif = floor((abs(oldquality-quality)/2.0)+0.5);
        if (osize > tsize) {
 	 newquality=quality-dif;
 	 if (dif < 1) { newquality--; searchdone=1; }
@@ -758,6 +768,7 @@ int main(int argc, char **argv)
 
    if (buf) FREE_LINE_BUF(buf,dinfo.output_height);
    jpeg_finish_decompress(&dinfo);
+   fclose(infile);
 
 
    if (quality>=0 && outsize>=insize && !retry && !stdin_mode) {
@@ -780,46 +791,50 @@ int main(int argc, char **argv)
 
 	if (stdout_mode) {
 	  outfname=NULL;
+	  set_filemode_binary(stdout);
 	  if (fwrite(outbuffer,outbuffersize,1,stdout) != 1)
 	    fatal("write failed to stdout");
 	} else {
+	  if (preserve_perms && !dest) {
+	    /* make backup of the original file */
+	    snprintf(tmpfilename,sizeof(tmpfilename),"%s.jpegoptim.bak",newname);
+	    if (verbose_mode > 1 && !quiet_mode) 
+	      fprintf(LOG_FH,"creating backup of original image as: %s\n",tmpfilename);
+	    if (file_exists(tmpfilename))
+	      fatal("backup file already exists: %s",tmpfilename);
+	    if (copy_file(newname,tmpfilename))
+	      fatal("failed to create backup of original file");
+	    if ((outfile=fopen(newname,"wb"))==NULL)
+	      fatal("error opening output file: %s", newname);
+	    outfname=newname;
+	  } else {
 #ifdef HAVE_MKSTEMPS
-          /* rely on mkstemps() to create us temporary file safely... */  
-	  snprintf(tmpfilename,sizeof(tmpfilename),
-		   "%sjpegoptim-%d-%d.XXXXXX.tmp", tmpdir, (int)getuid(), (int)getpid());
-	  if ((tmpfd = mkstemps(tmpfilename,4)) < 0) 
-	    fatal("error creating temp file: mkstemps() failed");
-	  if ((outfile=fdopen(tmpfd,"w"))==NULL) 
+	    /* rely on mkstemps() to create us temporary file safely... */  
+	    snprintf(tmpfilename,sizeof(tmpfilename),
+		     "%sjpegoptim-%d-%d.XXXXXX.tmp", tmpdir, (int)getuid(), (int)getpid());
+	    if ((tmpfd = mkstemps(tmpfilename,4)) < 0) 
+	      fatal("error creating temp file: mkstemps() failed");
+	    if ((outfile=fdopen(tmpfd,"wb"))==NULL) 
 #else
-	  /* if platform is missing mkstemps(), try to create at least somewhat "safe" temp file... */  
-	  snprintf(tmpfilename,sizeof(tmpfilename),
-		   "%sjpegoptim-%d-%d.%d.tmp", tmpdir, (int)getuid(), (int)getpid(),time(NULL));
-	  tmpfd=0;
-	  if ((outfile=fopen(tmpfilename,"w"))==NULL) 
+	      /* if platform is missing mkstemps(), try to create at least somewhat "safe" temp file... */  
+	      snprintf(tmpfilename,sizeof(tmpfilename),
+		       "%sjpegoptim-%d-%d.%d.tmp", tmpdir, (int)getuid(), (int)getpid(),time(NULL));
+	    tmpfd=0;
+	    if ((outfile=fopen(tmpfilename,"wb"))==NULL) 
 #endif
-	    fatal("error opening temporary file: %s",tmpfilename);
-	  outfname=tmpfilename;
-
+	      fatal("error opening temporary file: %s",tmpfilename);
+	    outfname=tmpfilename;
+	  }
 
 	  if (verbose_mode > 1 && !quiet_mode) 
-	    fprintf(LOG_FH,"writing %lu bytes to temporary file: %s\n",
+	    fprintf(LOG_FH,"writing %lu bytes to file: %s\n",
 		    (long unsigned int)outbuffersize, outfname);
 	  if (fwrite(outbuffer,outbuffersize,1,outfile) != 1)
-	    fatal("write failed to temporary file");
+	    fatal("write failed to file: %s", outfname);
 	  fclose(outfile);
 	}
 
 	if (outfname) {
-	  /* preserve file mode */
-	  if (chmod(outfname,(file_stat.st_mode & 0777)) != 0) 
-	    warn("failed to set output file mode"); 
-
-	  /* preserve file group (and owner if run by root) */
-	  if (chown(outfname,
-		    (geteuid()==0 ? file_stat.st_uid : -1),
-		    file_stat.st_gid) != 0)
-	    warn("failed to reset output file group/owner");
-
 	  
 	  if (preserve_mode) {
 	    /* preserve file modification time */
@@ -830,9 +845,27 @@ int main(int argc, char **argv)
 	      warn("failed to reset output file time/date");
 	  }
 
-	  if (verbose_mode > 1 && !quiet_mode) 
-	    fprintf(LOG_FH,"renaming: %s to %s\n",outfname,newname);
-	  if (rename_file(outfname,newname)) fatal("cannot rename temp file");
+	  if (preserve_perms && !dest) {
+	    /* original file was already replaced, remove backup... */
+	    if (delete_file(tmpfilename))
+	      warn("failed to remove backup file: %s",tmpfilename);
+	  } else {
+	    /* make temp file to be the original file... */
+
+	    /* preserve file mode */
+	    if (chmod(outfname,(file_stat.st_mode & 0777)) != 0) 
+	      warn("failed to set output file mode"); 
+
+	    /* preserve file group (and owner if run by root) */
+	    if (chown(outfname,
+		      (geteuid()==0 ? file_stat.st_uid : -1),
+		      file_stat.st_gid) != 0)
+	      warn("failed to reset output file group/owner");
+
+	    if (verbose_mode > 1 && !quiet_mode) 
+	      fprintf(LOG_FH,"renaming: %s to %s\n",outfname,newname);
+	    if (rename_file(outfname,newname)) fatal("cannot rename temp file");
+	  }
 	}
    } else {
      if (!quiet_mode || csv) fprintf(LOG_FH,csv ? "skipped\n" : "skipped.\n");
