@@ -1,6 +1,8 @@
 /* win32_compat.h
  *
  * compatibility stuff for Windows
+ *
+ * Thanks to Javier Gutiérrez Chamorro for Windows support.
  */
 
 #ifndef _WIN32_COMPAT_H
@@ -23,6 +25,8 @@ extern "C" {
 #define realpath(N,R) _fullpath((R),(N),MAXPATHLEN)
 #define ftruncate(fildes,length) open(fildes, O_TRUNC|O_WRONLY)
 
+#define set_filemode_binary(file) _setmode(_fileno(file), _O_BINARY)
+
 #define round(x) ((int) (x))
 #define getuid(x) 0
 #define geteuid() 0
